@@ -39,7 +39,20 @@
 
 const char *myname;
 
-void do_ck_fclose P_((FILE *stream));
+/* Store information about files opened with ck_fopen
+   so that error messages from ck_fread, ck_fwrite, etc. can print the
+   name of the file that had the error */
+
+struct id
+  {
+    FILE *fp;
+    char *name;
+    struct id *link;
+    unsigned temp : 1;
+  };
+
+static struct id *utils_id_s = NULL;
+static void do_ck_fclose P_((FILE *fp));
 
 /* Print an error message and exit */
 #if !defined __STDC__ || !(__STDC__-0)
@@ -71,23 +84,27 @@ panic(const char *str, ...)
 #endif /* HAVE_VFPRINTF */
   va_end(iggy);
   putc('\n', stderr);
+
+  /* Unlink the temporary files.  */
+  while (utils_id_s)
+    {
+      if (utils_id_s->temp)
+	{
+	  int fd = fileno (utils_id_s->fp);
+	  fclose (utils_id_s->fp);
+	  errno = 0;
+	  unlink (utils_id_s->name);
+          if (errno != 0)
+            fprintf (stderr, _("cannot remove %s: %s"), utils_id_s->name, strerror (errno));
+	}
+
+      utils_id_s = utils_id_s->link;
+    }
+
   exit(4);
 }
 
 
-/* Store information about files opened with ck_fopen
-   so that error messages from ck_fread, ck_fwrite, etc. can print the
-   name of the file that had the error */
-
-struct id
-  {
-    FILE *fp;
-    char *name;
-    struct id *link;
-  };
-
-static struct id *utils_id_s = NULL;
-
 /* Internal routine to get a filename from utils_id_s */
 static const char *utils_fp_name P_((FILE *fp));
 static const char *
@@ -114,7 +131,7 @@ FILE *
 ck_fopen(name, mode, fail)
   const char *name;
   const char *mode;
-  flagT fail;
+  bool fail;
 {
   FILE *fp;
   struct id *p;
@@ -144,6 +161,49 @@ ck_fopen(name, mode, fail)
     }
   p->name = ck_strdup(name);
   p->fp = fp;
+  p->temp = false;
+  return fp;
+}
+
+FILE *
+ck_mkstemp (p_filename, tmpdir, base)
+  char **p_filename;
+  char *base, *tmpdir;
+{
+  char *template;
+  FILE *fp;
+  int fd;
+  struct id *p;
+
+  if (tmpdir == NULL)
+    tmpdir = getenv("TMPDIR");
+  if (tmpdir == NULL)
+    {
+      tmpdir = getenv("TMP");
+      if (tmpdir == NULL)
+#ifdef P_tmpdir
+	tmpdir = P_tmpdir;
+#else
+	tmpdir = "/tmp";
+#endif
+    }
+
+  template = xmalloc (strlen (tmpdir) + strlen (base) + 8);
+  sprintf (template, "%s/%sXXXXXX", tmpdir, base);
+
+  fd = mkstemp (template);
+  if (fd == -1)
+    panic(_("Couldn't open temporary file %s: %s"), template, strerror(errno));
+
+  *p_filename = template;
+  fp = fdopen (fd, "w");
+
+  p = MALLOC(1, struct id);
+  p->name = ck_strdup (template);
+  p->fp = fp;
+  p->temp = true;
+  p->link = utils_id_s;
+  utils_id_s = p;
   return fp;
 }
 
@@ -177,6 +237,22 @@ ck_fread(ptr, size, nmemb, stream)
   return nmemb;
 }
 
+size_t
+ck_getline(text, buflen, stream)
+  char **text;
+  size_t *buflen;
+  FILE *stream;
+{
+  int result;
+  if (!ferror (stream))
+    result = getline (text, buflen, stream);
+
+  if (ferror (stream))
+    panic (_("read error on %s: %s"), utils_fp_name(stream), strerror(errno));
+
+  return result;
+}
+
 /* Panic on failing fflush */
 void
 ck_fflush(stream)
@@ -224,43 +300,49 @@ ck_fclose(stream)
     }
 }
 
-/* Close a single file and update a count of closed files. */
+/* Close a single file. */
 void
-do_ck_fclose(stream)
-  FILE *stream;
+do_ck_fclose(fp)
+  FILE *fp;
 {
-  ck_fflush(stream);
-  clearerr(stream);
-  if (fclose(stream) == EOF)
-    panic("Couldn't close %s: %s", utils_fp_name(stream), strerror(errno));
+  int fd;
+  ck_fflush(fp);
+  clearerr(fp);
+
+  /* We want to execute both arms, so use | not ||.  */
+  if (fclose(fp) == EOF)
+    panic("Couldn't close %s: %s", utils_fp_name(fp), strerror(errno));
 }
-
-
 
-char *
-temp_file_template(tmpdir, program)
-  const char *tmpdir;    
-  char *program;
+
+/* Panic on failing rename */
+void
+ck_rename (from, to, unlink_if_fail)
+  const char *from, *to;
+  const char *unlink_if_fail;
 {
-  char *template;
-  if (tmpdir == NULL)
-    tmpdir = getenv("TMPDIR");
-  if (tmpdir == NULL)
+  int rd = rename (from, to);
+  if (rd != -1)
+    return;
+
+  if (unlink_if_fail)
     {
-      tmpdir = getenv("TMP");
-      if (tmpdir == NULL)
-#ifdef P_tmpdir
-	tmpdir = P_tmpdir;
-#else
-	tmpdir = "/tmp";
-#endif
+      int save_errno = errno;
+      errno = 0;
+      unlink (unlink_if_fail);
+
+      /* Failure to remove the temporary file is more severe, so trigger it first.  */
+      if (errno != 0)
+        panic (_("cannot remove %s: %s"), unlink_if_fail, strerror (errno));
+
+      errno = save_errno;
     }
 
-  template = xmalloc (strlen (tmpdir) + strlen (program) + 8);
-  sprintf (template, "%s/%sXXXXXX", tmpdir, program);
-  return (template);
+  panic (_("cannot rename %s: %s"), from, strerror (errno));
 }
 
+
+
 
 /* Panic on failing malloc */
 VOID *
@@ -390,19 +472,21 @@ resize_buffer(b, newlen)
   b->b = try;
 }
 
-void
+char *
 add_buffer(b, p, n)
   struct buffer *b;
   const char *p;
   size_t n;
 {
+  char *result;
   if (b->allocated - b->length < n)
     resize_buffer(b, b->length+n);
-  memcpy(b->b + b->length, p, n);
+  result = memcpy(b->b + b->length, p, n);
   b->length += n;
+  return result;
 }
 
-void
+char *
 add1_buffer(b, c)
   struct buffer *b;
   int c;
@@ -415,10 +499,15 @@ add1_buffer(b, c)
    */
   if (c != EOF)
     {
+      char *result;
       if (b->allocated - b->length < 1)
 	resize_buffer(b, b->length+1);
-      b->b[b->length++] = c;
+      result = b->b + b->length++;
+      *result = c;
+      return result;
     }
+
+  return NULL;
 }
 
 void
