@@ -29,7 +29,7 @@
 # define N_(String) (String)
 #endif
 
-extern flagT use_extended_syntax_p;
+extern bool use_extended_syntax_p;
 
 static const char errors[] =
   "No previous regular expression\0"
@@ -102,7 +102,7 @@ compile_regex(b, flags, needed_sub)
       }
 
     /* GNU regex does not process \t & co. */
-    last_re_len = normalize_text(last_re, last_re_len);
+    last_re_len = normalize_text(last_re, last_re_len, TEXT_REGEX);
     re_set_syntax (syntax);
     error = re_compile_pattern (last_re, last_re_len, new_regex);
     new_regex->newline_anchor = (flags & REG_NEWLINE) != 0;
@@ -128,7 +128,7 @@ compile_regex(b, flags, needed_sub)
   FREE(last_re);
 
   /* Just to be sure, I mark this as not POSIXLY_CORRECT behavior */
-  if (new_regex->re_nsub < needed_sub && !POSIXLY_CORRECT)
+  if (new_regex->re_nsub < needed_sub && posixicity == POSIXLY_EXTENDED)
     {
       char buf[200];
       sprintf(buf, _("Invalid reference \\%d on `s' command's RHS"),
@@ -189,11 +189,9 @@ match_regex(regex, buf, buflen, buf_start_offset, regarray, regsize)
   int ret;
   static regex_t *regex_last;
 #ifdef REG_PERL
-  regmatch_t *regmatch;
-  if (regsize)
+  regmatch_t rm[10], *regmatch = rm;
+  if (regsize > 10)
     regmatch = (regmatch_t *) alloca (sizeof (regmatch_t) * regsize);
-  else
-    regmatch = NULL;
 #endif
 
   /* printf ("Matching from %d/%d\n", buf_start_offset, buflen); */
@@ -209,9 +207,9 @@ match_regex(regex, buf, buflen, buf_start_offset, regarray, regsize)
     regex_last = regex;
 
 #ifdef REG_PERL
-  ret = regexec2(regex, 
-		 buf, CAST(int)buflen, CAST(int)buf_start_offset,
-		 regsize, regmatch, 0);
+  regmatch[0].rm_so = CAST(int)buf_start_offset;
+  regmatch[0].rm_eo = CAST(int)buflen;
+  ret = regexec (regex, buf, regsize, regmatch, REG_STARTEND);
 
   if (regsize)
     copy_regs (regarray, regmatch, regsize);
@@ -219,9 +217,10 @@ match_regex(regex, buf, buflen, buf_start_offset, regarray, regsize)
   return (ret == 0);
 #else
   regex->regs_allocated = REGS_REALLOCATE;
-  ret = re_search(regex, buf, buflen, buf_start_offset,
-		  buflen - buf_start_offset,
-		  regsize ? regarray : NULL);
+
+  ret = re_search (regex, buf, buflen, buf_start_offset,
+		   buflen - buf_start_offset,
+		   regsize ? regarray : NULL);
 
   return (ret > -1);
 #endif
